
use std::sync::{Arc, atomic::{AtomicUsize, Ordering}};
use std::thread;

// shows that Ordering::SeqCst  and Ordering::Relaxed
// dont matter is all we do is one atomic op per thread per shared item

fn main() {
    // Number of threads and increments per thread
    let num_threads = 10;
    let increments_per_thread = 1000;

    // Using SeqCst
    let counter_seq = Arc::new(AtomicUsize::new(0));
    let handles_seq: Vec<_> = (0..num_threads).map(|_| {
        let counter = Arc::clone(&counter_seq);
        thread::spawn(move || {
            for _ in 0..increments_per_thread {
                counter.fetch_add(1, Ordering::SeqCst);
            }
        })
    }).collect();

    for handle in handles_seq {
        handle.join().unwrap();
    }

    println!("Final count with SeqCst: {}", counter_seq.load(Ordering::SeqCst));

    // Using Relaxed
    let counter_relaxed = Arc::new(AtomicUsize::new(0));
    let handles_relaxed: Vec<_> = (0..num_threads).map(|_| {
        let counter = Arc::clone(&counter_relaxed);
        thread::spawn(move || {
            for _ in 0..increments_per_thread {
                counter.fetch_add(1, Ordering::Relaxed);
            }
        })
    }).collect();

    for handle in handles_relaxed {
        handle.join().unwrap();
    }

    println!("Final count with Relaxed: {}", counter_relaxed.load(Ordering::Relaxed));
}

