
use std::io;
use std::sync::mpsc;
use std::thread;

// Define an enum to represent different message types
enum Message {
    Float(f64),
    String(String),
    Terminate,
}

fn main() {
    let (tx1, rx1): (mpsc::Sender<Message>, mpsc::Receiver<Message>) = mpsc::channel();
    let (tx2, rx2): (mpsc::Sender<Message>, mpsc::Receiver<Message>) = mpsc::channel();

    // Thread t1: Handles Float messages and forwards others to t2
    let t1 = thread::spawn(move || {
        while let Ok(message) = rx1.recv() {
            match message {
                Message::Float(num) => {
                    println!("(T1) Square root of {}: {}", num, num.sqrt());
                }
                Message::String(msg) => {
                    tx2.send(Message::String(msg)).unwrap();
                }
                Message::Terminate => {
                    // Forward Terminate to t2 and exit
                    tx2.send(Message::Terminate).unwrap();
                    break;
                }
            }
        }
    });

    // Thread t2: Handles specific String messages
    let t2 = thread::spawn(move || {
        while let Ok(message) = rx2.recv() {
            match message {
                Message::String(msg) => {
                    if msg.len() >= 3 && msg.chars().take(3).all(|c| c == msg.chars().next().unwrap()) {
                        println!("(T2) got one: {}", msg);
                    } else {
                        println!("(EOC) message not handled: {}", msg);
                    }
                }
                Message::Terminate => {
                    // Exit signal received
                    break;
                }
                _ => unreachable!(), // t2 only handles String and Terminate
            }
        }
    });

    // Main thread: Reads user input and classifies it into Message variants
    for ix in 0..5 {
        println!("{ix}: Enter a message:");
        let mut input = String::new();
        io::stdin().read_line(&mut input).expect("Failed to read line");
        let input = input.trim().to_string();

        // Centralized parsing of input
        if let Ok(num) = input.parse::<f64>() {
            tx1.send(Message::Float(num)).unwrap();
        } else {
            tx1.send(Message::String(input)).unwrap();
        }
    }

    // Signal termination by sending Terminate
    tx1.send(Message::Terminate).unwrap();

    // Wait for threads to finish
    t1.join().unwrap();
    t2.join().unwrap();
}

