use std::io;
use std::sync::mpsc;
use std::thread;

fn main() {
    // Channels use Option<String> for termination signaling
    let (tx1, rx1): (mpsc::Sender<Option<String>>, mpsc::Receiver<Option<String>>) = mpsc::channel();
    let (tx2, rx2): (mpsc::Sender<Option<String>>, mpsc::Receiver<Option<String>>) = mpsc::channel();

    // Thread t1: Handles floats and passes unhandled messages to t2
    let t1 = thread::spawn(move || {
        while let Ok(message) = rx1.recv() {
            match message {
                Some(msg) => {
                    if let Ok(num) = msg.parse::<f64>() {
                        println!("(T1) Square root of {}: {}", num, num.sqrt());
                    } else {
                        tx2.send(Some(msg)).unwrap();
                    }
                }
                None => {
                    // Forward None to t2 and exit
                    tx2.send(None).unwrap();
                    break;
                }
            }
        }
    });

    // Thread t2: Handles strings of a specific format
    let t2 = thread::spawn(move || {
        while let Ok(message) = rx2.recv() {
            match message {
                Some(msg) => {
                    if msg.len() >= 3 && msg.chars().take(3).all(|c| c == msg.chars().next().unwrap()) {
                        println!("(T2) got one: {}", msg);
                    } else {
                        println!("(EOC) message not handled: {}", msg);
                    }
                }
                None => {
                    // Exit signal received
                    break;
                }
            }
        }
    });

    // Main thread: Reads user input and sends it to t1
    for i in 0..5 {
        println!("{i}: Enter a message:");
        let mut input = String::new();
        io::stdin().read_line(&mut input).expect("Failed to read line");
        let input = input.trim().to_string();
        tx1.send(Some(input)).unwrap();
    }

    // Signal termination by sending None
    tx1.send(None).unwrap();

    // Wait for threads to finish
    t1.join().unwrap();
    t2.join().unwrap();
}


