use std::sync::mpsc;
use std::thread;
use std::time::Duration;

fn main() {
    let (ping_tx, ping_rx) = mpsc::channel::<String>();
    let (pong_tx, pong_rx) = mpsc::channel::<String>();

    // Spawn the "ping" thread with a named function
    let ping_tx_clone = ping_tx.clone(); // Clone the sender for `ping`
    let ping_handle = thread::spawn(move || {
        ping(ping_tx_clone, pong_rx);
    });

    // Spawn the "pong" thread with a named function
    let pong_handle = thread::spawn(move || {
        pong(ping_rx, pong_tx);
    });

    // Wait for both threads to finish
    ping_handle.join().unwrap();
    pong_handle.join().unwrap();
}

// The "ping" function
fn ping(ping_tx: mpsc::Sender<String>, pong_rx: mpsc::Receiver<String>) {
    for _ in 0..10 {
        thread::sleep(Duration::from_secs(1));
        // Send "Ping" to the pong thread

        ping_tx.send("Ping".to_string()).unwrap();

        thread::sleep(Duration::from_millis(500));

        // Receive "Pong" response
        if let Ok(msg) = pong_rx.recv() {
            println!("Ping received: {}", msg);
        }
    }
}

// The "pong" function
fn pong(ping_rx: mpsc::Receiver<String>, pong_tx: mpsc::Sender<String>) {
    for _ in 0..10 {
        thread::sleep(Duration::from_secs(1));
        // Receive "Ping" message
        if let Ok(msg) = ping_rx.recv() {
            println!("Pong received: {}", msg);

            thread::sleep(Duration::from_millis(500));

            // Send "Pong" back to the ping thread
            pong_tx.send("Pong".to_string()).unwrap();
        }
    }
}

